% Q: Column based matrix of eigenvectors
% lambda: vector of eigenvalues
function [Q, lambda] = energyEigensys2D(v, k, theta_r)
    v1 = v(1:2);
    v2 = v(3:4);
    P = [0, 1; -1, 0];
    v1perp = P * v1;
    v2perp = P * v2;
    dot1 = dot(v1, v1);
    dot2 = dot(v2, v2);
    [theta, s] = hingeAngleCrossSign(v);
    % these derivatives varies with different forms of the energy
    pPhipTheta = k * (theta - theta_r);
    p2PhipTheta2 = k;
    % compute the constants
    n1 = p2PhipTheta2 / dot1; n2 = p2PhipTheta2 / dot2;
    w = pPhipTheta / p2PhipTheta2 * s;
    w2 = w.^2;
    alpha = ((n1 - n2)/(n1 + n2)).^2;
    r = sqrt(4 * w2 * alpha + 1) * (n1 + n2);
    Rminus = sqrt(2 * (2 * w2 + 1 - r/(n1 + n2))) * (n1 + n2);
    Rplus = sqrt(2 * (2 * w2 + 1 + r/(n1 + n2))) * (n1 + n2);
    % compute the eigenvalues
    lambda0 = -r - Rminus + n1 + n2;
    lambda1 = -r + Rminus + n1 + n2;
    lambda2 = r - Rplus + n1 + n2;
    lambda3 = r + Rplus + n1 + n2;
    lambda = 0.25 * [lambda0;lambda1;lambda2;lambda3];
    % compute the eigenvectors in the basis of the edge vectors
    % orthogonalized but not normalized
    q0 = zeros(4, 1); q1 = q0; q2 = q0; q3 = q0;
    q0(1) = w/n2*((n1-n2).^2 - (n1+n2) * r - (n1-n2) * Rminus);
    q0(2) = ((n1+n2)*(-n1-2*w2*(n1-n2))+n1*r+0.5*(n1-n2-r)*Rminus)/n2;
    q0(3) = 4 * n2 * w;
    q0(4) = (n1+n2) - r - Rminus;
    q1(1) = w/n2*((n1-n2).^2 - (n1+n2) * r + (n1-n2) * Rminus);
    q1(2) = ((n1+n2)*(-n1-2*w2*(n1-n2))+n1*r-0.5*(n1-n2-r)*Rminus)/n2;
    q1(3) = 4 * n2 * w;
    q1(4) = (n1+n2) - r + Rminus;
    q2(1) = w/n2*((n1-n2).^2 + (n1+n2) * r - (n1-n2) * Rplus);
    q2(2) = ((n1+n2)*(-n1-2*w2*(n1-n2))-n1*r+0.5*(n1-n2+r)*Rplus)/n2;
    q2(3) = 4 * n2 * w;
    q2(4) = (n1+n2) + r - Rplus;
    q3(1) = w/n2*((n1-n2).^2 + (n1+n2) * r + (n1-n2) * Rplus);
    q3(2) = ((n1+n2)*(-n1-2*w2*(n1-n2))-n1*r-0.5*(n1-n2+r)*Rplus)/n2;
    q3(3) = 4 * n2 * w;
    q3(4) = (n1+n2) + r + Rplus;
    % compute the eigenvectors by applying the change of basis
    A = [v1,v1perp,zeros(2,1),zeros(2,1);
        zeros(2,1),zeros(2,1),v2, v2perp];
    u0 = A * q0; u0 = u0/norm(u0);
    u1 = A * q1; u1 = u1/norm(u1);
    u2 = A * q2; u2 = u2/norm(u2);
    u3 = A * q3; u3 = u3/norm(u3);
    Q = [u0, u1, u2, u3];
end