% Q: Column based matrix of eigenvectors
% lambda: vector of eigenvalues
% F: 6x1
function [Q, lambda] = energyEigensys3DCloth(F, k, theta_r, s)
    e0 = F(1:3); e1 = F(4:6); 
    crs = cross(e0, e1);
    cnorm = norm(crs);
    n = crs / cnorm;
    e0perp = cross(e0, n);
    e1perp = cross(e1, n);
    dot0 = dot(e0, e0); dot1 = dot(e1, e1);
    norm0 = norm(e0); norm1 = norm(e1);
    % the derivatives of the energy with respect to theta
    theta = s * (pi - acos(dot(e0, e1)/norm0/norm1));
    g = k * (theta - theta_r); h = k;
    % the 2 consisting of n. Similar with the eigensys of theta.
    beta = -(norm1/norm0 - norm0/norm1) * cos(theta);
    alpha = (- beta + sqrt(beta * beta + 4)) * 0.5;
    q4 = [alpha * n;n];
    lambda4 = -g*(-1/dot1/tan(theta) - alpha/norm0/norm1/sin(theta));
    q5 = [n;-alpha * n];
    lambda5 = -g*(-1/dot1/tan(theta) + 1/alpha/norm0/norm1/sin(theta));
    % the 4 analogous to 2D.
    % this is basically copying the code from 2D.

    % compute the constants
    n0 = h / dot0; n1 = h / dot1;
    w = -s*g / h;
    w2 = w.^2;
    alpha = ((n0 - n1)/(n0 + n1)).^2;
    r = sqrt(4 * w2 * alpha + 1) * (n0 + n1);
    Rminus = sqrt(2 * (2 * w2 + 1 - r/(n0 + n1))) * (n0 + n1);
    Rplus = sqrt(2 * (2 * w2 + 1 + r/(n0 + n1))) * (n0 + n1);
    % compute the eigenvalues
    lambda0 = -r - Rminus + n0 + n1;
    lambda1 = -r + Rminus + n0 + n1;
    lambda2 = r - Rplus + n0 + n1;
    lambda3 = r + Rplus + n0 + n1;
    lambda0123 = 0.25 * [lambda0;lambda1;lambda2;lambda3];
    % compute the eigenvectors in the basis of the edge vectors
    % orthogonalized but not normalized
    q0 = zeros(4, 1); q1 = q0; q2 = q0; q3 = q0;
    q0(1) = w/n1*((n0-n1).^2 - (n0+n1) * r - (n0-n1) * Rminus);
    q0(2) = ((n0+n1)*(-n0-2*w2*(n0-n1))+n0*r+0.5*(n0-n1-r)*Rminus)/n1;
    q0(3) = 4 * n1 * w;
    q0(4) = (n0+n1) - r - Rminus;
    q1(1) = w/n1*((n0-n1).^2 - (n0+n1) * r + (n0-n1) * Rminus);
    q1(2) = ((n0+n1)*(-n0-2*w2*(n0-n1))+n0*r-0.5*(n0-n1-r)*Rminus)/n1;
    q1(3) = 4 * n1 * w;
    q1(4) = (n0+n1) - r + Rminus;
    q2(1) = w/n1*((n0-n1).^2 + (n0+n1) * r - (n0-n1) * Rplus);
    q2(2) = ((n0+n1)*(-n0-2*w2*(n0-n1))-n0*r+0.5*(n0-n1+r)*Rplus)/n1;
    q2(3) = 4 * n1 * w;
    q2(4) = (n0+n1) + r - Rplus;
    q3(1) = w/n1*((n0-n1).^2 + (n0+n1) * r + (n0-n1) * Rplus);
    q3(2) = ((n0+n1)*(-n0-2*w2*(n0-n1))-n0*r-0.5*(n0-n1+r)*Rplus)/n1;
    q3(3) = 4 * n1 * w;
    q3(4) = (n0+n1) + r + Rplus;
    % compute the eigenvectors by applying the change of basis
    A = [e0,e0perp,zeros(3,1),zeros(3,1);
        zeros(3,1),zeros(3,1),e1, e1perp]; % 6x4
    q0 = A * q0; 
    q1 = A * q1; 
    q2 = A * q2; 
    q3 = A * q3; 

    % normalize 
    q0 = q0/norm(q0); q1 = q1/norm(q1); q2 = q2/norm(q2);
    q3 = q3/norm(q3); q4 = q4/norm(q4); q5 = q5/norm(q5);
    Q = [q0,q1,q2,q3,q4,q5]; 
    lambda = [lambda0123;lambda4;lambda5];
end