% Using finite difference to compute the Hessian of f
% at position pos.
% delta: finite differece delta
function h = energyHessian(f, pos, k, theta_r, delta)
    n = length(pos);
    [X, Y] = meshgrid(1:n, 1:n);
    func = @(ii, jj) energyHessian1D(f, pos, k, theta_r, delta, ii, jj, n);
    h = arrayfun(func, X, Y);
end

function g = energyHessian1D(f, pos, k, theta_r, delta, ii, jj, n)
    zi = zeros(n,1);
    zj = zeros(n,1);
    zi(ii) =  delta;
    zj(jj) = delta;
    g = (f(pos + zi + zj, k, theta_r) + f(pos - zi - zj, k, theta_r)...
        - f(pos - zi + zj, k, theta_r) - f(pos + zi - zj, k, theta_r))...
        /4/delta/delta;
end