function h = hessianAnalyticalDG(v, k, theta_r)
    perp = [0 1; -1 0];
    v1 = v(1:2);
    v2 = v(3:4);
    [theta, s] = hingeAngleDG(v1, v2);
    dot1 = dot(v1, v1);
    dot2 = dot(v2, v2);
    pThetaPV1 = perp*v1/dot1 * s;
    pThetaPV2 = -perp*v2/dot2 * s;
    p2ThetaPV12 = (perp * dot1 - 2 * perp * (v1 * v1.'))/dot1/dot1 * s;
    p2ThetaPV22 = -(perp * dot2 - 2 * perp * (v2 * v2.'))/dot2/dot2 * s;
    block1 = k * (pThetaPV1 * pThetaPV1.') +...
        k * (theta - theta_r) * p2ThetaPV12;
    block2 = k * (pThetaPV2 * pThetaPV2.') +...
        k * (theta - theta_r) * p2ThetaPV22;
    blockz = k * (pThetaPV1 * pThetaPV2.');
    h = [block1 blockz; blockz.' block2];
end

function [agl, s] = hingeAngleDG(v1, v2)
    v1_3 = [v1; 0];
    v2_3 = [v2; 0];
    cr = cross(v1_3, v2_3);
    c = cr(3);
    s = sign(c);
    tangent = abs(c)/dot(v1_3, v2_3);
    agl = atanNew(tangent);
end

function agl = atanNew(x)
    temp = atan(x);
    if temp < 0
        agl = temp + pi;
    else
        agl = temp;
    end
end