% Q: Column based matrix of eigenvectors
% lambda: vector of eigenvalues
% F: 6x1
function [Q, lambda] = thetaEigensys3D(F, ~, ~)
    e0 = F(1:3); e1 = F(4:6); 
    crs = cross(e0, e1);
    cnorm = norm(crs);
    n = crs / cnorm;
    e0perp = cross(e0, n);
    e1perp = cross(e1, n);
    dot0 = dot(e0, e0); dot1 = dot(e1, e1);
    norm0 = norm(e0); norm1 = norm(e1);
    % the first 4
    q0 = [e0+e0perp;zeros(3,1)]; lambda0 = -1/dot0;
    q1 = [e0-e0perp;zeros(3,1)]; lambda1 = 1/dot0;
    q2 = [zeros(3,1);e1+e1perp]; lambda2 = 1/dot1;
    q3 = [zeros(3,1);e1-e1perp]; lambda3 = -1/dot1;
    % the other 2
    theta = hingeAngleCos3D(F);
    beta = (norm1/norm0 - norm0/norm1) * cos(theta);
    alpha = (- beta + sqrt(beta * beta + 4)) * 0.5;
    q4 = [alpha * n;n];
    lambda4 = 1/dot1/tan(theta) - alpha/norm0/norm1/sin(theta);
    q5 = [n;-alpha * n];
    lambda5 = 1/dot1/tan(theta) + 1/alpha/norm0/norm1/sin(theta);
    % normalize 
    q0 = q0/norm(q0); q1 = q1/norm(q1); q2 = q2/norm(q2);
    q3 = q3/norm(q3); q4 = q4/norm(q4); q5 = q5/norm(q5);
    Q = [q0,q1,q2,q3,q4,q5]; 
    lambda = [lambda0;lambda1;lambda2;lambda3;lambda4;lambda5];
end