% Compute the hessian of theta analytically
% Returns a 6x1 vector
% F: 6x1
function H = thetaHessian3D(F, ~, ~)
    e0 = F(1:3); e1 = F(4:6); 
    crs = cross(e0, e1);
    cnorm = norm(crs);
    n = crs / cnorm;
    e0perp = cross(e0, n);
    e1perp = cross(e1, n);
    N = getCrossMatrix(n);
    E0 = getCrossMatrix(e0);
    E1 = getCrossMatrix(e1);
    dot0 = dot(e0, e0);
    dot1 = dot(e1, e1);
    % derivatives of n
    pnpe0 = (E1 * cnorm - crs * e1perp.')/cnorm/cnorm;
    pnpe1 = (-E0 * cnorm + crs * e0perp.')/cnorm/cnorm;
    % compute the blocks
    B00 = (dot0 * N - 2 * e0perp * e0.' + (-E0 * pnpe0) * dot0)/dot0/dot0;
    B01 = -E0 * pnpe1 / dot0;
    B11 = (-dot1 * N + 2 * e1perp * e1.' + (E1 * pnpe1) * dot1)/dot1/dot1;
    B10 = E1 * pnpe0 / dot1;
    H = [B00 B01; B10 B11];
end