% Compute the hessian of theta analytically
% Returns a 6x1 vector
% F: 6x1
function H = thetaHessian3DCloth(F, ~, ~)
    e0 = F(1:3); e1 = F(4:6); 
    tau0 = e0/norm(e0); tau1 = e1/norm(e1);
    crs = cross(e0, e1);
    cnorm = norm(crs);
    n = crs / cnorm;
    crsProd = norm(cross(tau0, tau1));
    tau0perp = cross(tau0, n);
    tau1perp = cross(tau1, n);
    Tb = getCrossMatrix(n);
    T0 = getCrossMatrix(tau0);
    T1 = getCrossMatrix(tau1);
    dot0 = dot(e0, e0);
    dot1 = dot(e1, e1);
    normz0 = sqrt(dot0);
    normz1 = sqrt(dot1);
    % blocks
    B00 = 1/normz0/normz0 *(Tb - 2.0 * tau0perp * tau0.'...
                  -(T0 * T1 + tau0perp*tau1perp.')/crsProd);
    B01 = 1/crsProd/normz0/normz1 * (T0 * T0 + tau0perp*tau0perp.');
    B11 = 1/normz1/normz1 *(-Tb + 2.0 * tau1perp * tau1.'...
                  -(T1 * T0 + tau1perp*tau0perp.')/crsProd);
    B10 = 1/crsProd/normz0/normz1 * (T1 * T1 + tau1perp*tau1perp.');
    H = [B00 B01; B10 B11];
end